@@ -48,13 +48,16 @@
 #include "gdm-server.h"
 #include "gdm-chooser-server.h"
 #include "gdm-chooser-session.h"
+#include "gdm-settings-direct.h"
+#include "gdm-settings-keys.h"
 
 #define GDM_XDMCP_CHOOSER_SLAVE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_XDMCP_CHOOSER_SLAVE, GdmXdmcpChooserSlavePrivate))
 
 #define GDM_DBUS_NAME              "org.gnome.DisplayManager"
 #define GDM_DBUS_DISPLAY_INTERFACE "org.gnome.DisplayManager.Display"
 
-#define MAX_CONNECT_ATTEMPTS 10
+#define MAX_CONNECT_ATTEMPTS  10
+#define DEFAULT_PING_INTERVAL 15
 
 struct GdmXdmcpChooserSlavePrivate
 {
@@ -164,6 +167,7 @@ run_chooser (GdmXdmcpChooserSlave *slave)
         char          *display_hostname;
         char          *auth_file;
         char          *address;
+        gboolean       res;
 
         g_debug ("GdmXdmcpChooserSlave: Running chooser");
 
@@ -185,12 +189,16 @@ run_chooser (GdmXdmcpChooserSlave *slave)
         /* FIXME: send a signal back to the master */
 
         /* If XDMCP setup pinging */
+        slave->priv->ping_interval = DEFAULT_PING_INTERVAL;
+        res = gdm_settings_direct_get_int (GDM_KEY_PING_INTERVAL,
+                                           &(slave->priv->ping_interval));
+
         if (slave->priv->ping_interval > 0) {
                 alarm (slave->priv->ping_interval);
         }
 
         /* Run the init script. gdmslave suspends until script has terminated */
-        gdm_slave_run_script (GDM_SLAVE (slave), GDMCONFDIR "/Init", "gdm");
+        gdm_slave_run_script (GDM_SLAVE (slave), GDMCONFDIR "/Init", GDM_USERNAME);
 
         slave->priv->chooser_server = gdm_chooser_server_new (display_id);
         g_signal_connect (slave->priv->chooser_server,
